@@ -11,6 +11,7 @@ import logging
 from os import path as osp
 import os
 import shutil
+import subprocess
 import sys
 import time
 
@@ -18,8 +19,8 @@ from tornado.ioloop import IOLoop
 from tornado.iostream import StreamClosedError
 from tornado.websocket import WebSocketClosedError
 
-from notebook.notebookapp import flags, aliases
-from notebook.utils import urljoin, pathname2url
+from jupyter_server.serverapp import flags, aliases
+from jupyter_server.utils import urljoin, pathname2url
 from traitlets import Bool
 
 from .labapp import LabApp, get_app_dir
@@ -68,9 +69,7 @@ class LogErrorHandler(logging.Handler):
 
 def run_test(app, func):
     """Synchronous entry point to run a test function.
-
     func is a function that accepts an app url as a parameter and returns a result.
-
     func can be synchronous or asynchronous.  If it is synchronous, it will be run
     in a thread, so asynchronous is preferred.
     """
@@ -79,9 +78,7 @@ def run_test(app, func):
 
 async def run_test_async(app, func):
     """Run a test against the application.
-
     func is a function that accepts an app url as a parameter and returns a result.
-
     func can be synchronous or asynchronous.  If it is synchronous, it will be run
     in a thread, so asynchronous is preferred.
     """
@@ -91,6 +88,8 @@ async def run_test_async(app, func):
     env_patch = TestEnv()
     env_patch.start()
 
+    app.log.info('Running async test')
+
     # The entry URL for browser tests is different in notebook >= 6.0,
     # since that uses a local HTML file to point the user at the app.
     if hasattr(app, 'browser_open_file'):
@@ -129,7 +128,7 @@ async def run_test_async(app, func):
         app.io_loop.stop()
         env_patch.stop()
     except Exception as e:
-        self.log.error(str(e))
+        app.log.error(str(e))
         result = 1
     finally:
         time.sleep(2)
@@ -143,7 +142,7 @@ async def run_async_process(cmd, **kwargs):
             **kwargs)
     stdout, stderr = await proc.communicate()
     if proc.returncode != 0:
-        raise RuntimeError(cmd + ' exited with ' + str(proc.returncode))
+        raise RuntimeError(str(cmd) + ' exited with ' + str(proc.returncode))
     return stdout, stderr
 
 
@@ -152,34 +151,80 @@ async def run_browser(url):
     """
     target = osp.join(get_app_dir(), 'browser_test')
     if not osp.exists(osp.join(target, 'node_modules')):
-        os.makedirs(target)
+        if not osp.exists(target):
+            os.makedirs(osp.join(target))
         await run_async_process(["jlpm", "init", "-y"], cwd=target)
-        await run_async_process(["jlpm", "add", "puppeteer@^2"], cwd=target)
+        await run_async_process(["jlpm", "add", "puppeteer@^4"], cwd=target)
     shutil.copy(osp.join(here, 'chrome-test.js'), osp.join(target, 'chrome-test.js'))
     await run_async_process(["node", "chrome-test.js", url], cwd=target)
 
 
+def run_browser_sync(url):
+    """Run the browser test and return an exit code.
+    """
+    target = osp.join(get_app_dir(), 'browser_test')
+    if not osp.exists(osp.join(target, 'node_modules')):
+        os.makedirs(target)
+        subprocess.call(["jlpm", "init", "-y"], cwd=target)
+        subprocess.call(["jlpm", "add", "puppeteer@^2"], cwd=target)
+    shutil.copy(osp.join(here, 'chrome-test.js'), osp.join(target, 'chrome-test.js'))
+    return subprocess.check_call(["node", "chrome-test.js", url], cwd=target)
+
 class BrowserApp(LabApp):
     """An app the launches JupyterLab and waits for it to start up, checking for
     JS console errors, JS errors, and Python logged errors.
     """
-    open_browser = Bool(False)
-    base_url = '/foo/'
+    name = __name__
+    serverapp_config = {
+        "open_browser": False,
+        "base_url": "/foo/"
+    }
     ip = '127.0.0.1'
     flags = test_flags
     aliases = test_aliases
     test_browser = Bool(True)
 
-    def start(self):
-        web_app = self.web_app
-        self.kernel_manager.shutdown_wait_time = 1
+    def initialize_settings(self):
+        self.settings.setdefault('page_config_data', dict())
+        self.settings['page_config_data']['browserTest'] = True
+        self.settings['page_config_data']['buildAvailable'] = False
+        self.settings['page_config_data']['exposeAppInBrowser'] = True
+        super().initialize_settings()
 
-        web_app.settings.setdefault('page_config_data', dict())
-        web_app.settings['page_config_data']['browserTest'] = True
-        web_app.settings['page_config_data']['buildAvailable'] = False
+    def initialize_handlers(self):
         func = run_browser if self.test_browser else lambda url: 0
-        run_test(self, func)
-        super().start()
+        if os.name == 'nt' and func == run_browser:
+            func = run_browser_sync
+        run_test(self.serverapp, func)
+        super().initialize_handlers()
+
+
+def _jupyter_server_extension_points():
+        return [
+            {
+                'module': __name__,
+                'app': BrowserApp
+            }
+        ]
+
+
+# TODO: remove handling of --notebook arg and the following two
+# functions in JupyterLab 4.0
+def load_jupyter_server_extension(serverapp):
+    extension = BrowserApp()
+    extension.serverapp = serverapp
+    extension.load_config_file()
+    extension.update_config(serverapp.config)
+    extension.parse_command_line(serverapp.extra_args)
+    extension.initialize()
+
+
+def _jupyter_server_extension_paths():
+    return [
+        {
+            'module': 'jupyterlab.browser_check'
+        }
+    ]
 
 
 if __name__ == '__main__':
@@ -187,4 +232,13 @@ if __name__ == '__main__':
     if skip_option in sys.argv:
         BrowserApp.test_browser = False
         sys.argv.remove(skip_option)
-    BrowserApp.launch_instance()
+
+    if "--notebook" in sys.argv:
+        from notebook.notebookapp import NotebookApp
+        NotebookApp.default_url = "/lab"
+        sys.argv.remove("--notebook")
+        NotebookApp.nbserver_extensions = {"jupyterlab.browser_check": True}
+        NotebookApp.open_browser = False
+        NotebookApp.launch_instance()
+    else:
+        BrowserApp.launch_instance()
